@@ -2,7 +2,7 @@
 
   dln.c -
 
-  $Author: nagachika $
+  $Author: usa $
   created at: Tue Jan 18 17:05:06 JST 1994
 
   Copyright (C) 1993-2007 Yukihiro Matsumoto
@@ -107,10 +107,11 @@ dln_loaderror(const char *format, ...)
 
 #ifndef FUNCNAME_PATTERN
 # if defined(__hp9000s300) || ((defined(__NetBSD__) || defined(__FreeBSD__) || defined(__OpenBSD__)) && !defined(__ELF__)) || defined(__BORLANDC__) || defined(NeXT) || defined(__WATCOMC__) || defined(MACOSX_DYLD)
-#  define FUNCNAME_PREFIX "_Init_"
+#  define EXTERNAL_PREFIX "_"
 # else
-#  define FUNCNAME_PREFIX "Init_"
+#  define EXTERNAL_PREFIX ""
 # endif
+# define FUNCNAME_PREFIX EXTERNAL_PREFIX"Init_"
 #endif
 
 #if defined __CYGWIN__ || defined DOSISH
@@ -1340,6 +1341,24 @@ dln_load(const char *file)
 	    goto failed;
 	}
 
+# if defined RUBY_EXPORT
+	{
+	    static const char incompatible[] = "incompatible library version";
+	    void *ex = dlsym(handle, EXTERNAL_PREFIX"ruby_xmalloc");
+	    if (ex && ex != ruby_xmalloc) {
+
+#   if defined __APPLE__
+		/* dlclose() segfaults */
+		rb_fatal("%s - %s", incompatible, file);
+#   else
+		dlclose(handle);
+		error = incompatible;
+		goto failed;
+#   endif
+	    }
+	}
+# endif
+
 	init_fct = (void(*)())(VALUE)dlsym(handle, buf);
 #ifdef __native_client__
 	strcpy(file, orig);
