@@ -2,7 +2,7 @@
 
   dbm.c -
 
-  $Author$
+  $Author: nobu $
   created at: Mon Jan 24 15:59:52 JST 1994
 
   Copyright (C) 1995-2001 Yukihiro Matsumoto
