@@ -1,7 +1,8 @@
 /* $RoughId: rmd160init.c,v 1.3 2001/07/13 20:00:43 knu Exp $ */
-/* $Id: rmd160init.c 19020 2008-09-01 07:48:53Z nobu $ */
+/* $Id: rmd160init.c 52797 2015-11-30 12:08:16Z usa $ */
 
-#include "digest.h"
+#include <ruby/ruby.h>
+#include "../digest.h"
 #if defined(HAVE_OPENSSL_RIPEMD_H)
 #include "rmd160ossl.h"
 #else
@@ -30,6 +31,9 @@ Init_rmd160()
 
     rb_require("digest");
 
+#if 0
+    mDigest = rb_define_module("Digest"); /* let rdoc know */
+#endif
     mDigest = rb_path2class("Digest");
     cDigest_Base = rb_path2class("Digest::Base");
 
