@@ -2,50 +2,52 @@ require 'test/unit'
 require 'timeout'
 require 'tmpdir'
 require 'tempfile'
+require 'fileutils'
 require_relative 'envutil'
 
 class TestArgf < Test::Unit::TestCase
   def setup
-    @t1 = Tempfile.new("argf-foo")
+    @tmpdir = Dir.mktmpdir
+    @tmp_count = 0
+    @t1 = make_tempfile0("argf-foo")
     @t1.binmode
     @t1.puts "1"
     @t1.puts "2"
     @t1.close
-    @t2 = Tempfile.new("argf-bar")
+    @t2 = make_tempfile0("argf-bar")
     @t2.binmode
     @t2.puts "3"
     @t2.puts "4"
     @t2.close
-    @t3 = Tempfile.new("argf-baz")
+    @t3 = make_tempfile0("argf-baz")
     @t3.binmode
     @t3.puts "5"
     @t3.puts "6"
     @t3.close
-    @tmps = [@t1, @t2, @t3]
   end
 
   def teardown
-    @tmps.each {|t|
-      bak = t.path + ".bak"
-      File.unlink bak if File.file? bak
-      t.close(true)
-    }
+    FileUtils.rmtree(@tmpdir)
+  end
+
+  def make_tempfile0(basename)
+    @tmp_count += 1
+    open("#{@tmpdir}/#{basename}-#{@tmp_count}", "w")
   end
 
   def make_tempfile
-    t = Tempfile.new("argf-qux")
+    t = make_tempfile0("argf-qux")
     t.puts "foo"
     t.puts "bar"
     t.puts "baz"
     t.close
-    @tmps << t
     t
   end
 
-  def ruby(*args)
+  def ruby(*args, external_encoding: Encoding::UTF_8)
     args = ['-e', '$>.write($<.read)'] if args.empty?
     ruby = EnvUtil.rubybin
-    f = IO.popen([ruby] + args, 'r+')
+    f = IO.popen([ruby] + args, 'r+', external_encoding: external_encoding)
     yield(f)
   ensure
     f.close unless !f || f.closed?
@@ -201,21 +203,14 @@ class TestArgf < Test::Unit::TestCase
     t = make_tempfile
 
     assert_in_out_err(["-", t.path], <<-INPUT) do |r, e|
-      ARGF.inplace_mode = '/\\\\'
+      ARGF.inplace_mode = '/\\\\:'
       while line = ARGF.gets
         puts line.chomp + '.new'
       end
     INPUT
-      if no_safe_rename
-        assert_equal([], e)
-        assert_equal([], r)
-        assert_equal("foo.new\nbar.new\nbaz.new\n", File.read(t.path))
-        File.unlink(t.path + ".~~~") rescue nil
-      else
-        assert_match(/Can't rename .* to .*: .*. skipping file/, e.first) #'
-        assert_equal([], r)
-        assert_equal("foo\nbar\nbaz\n", File.read(t.path))
-      end
+      assert_match(/Can't rename .* to .*: .*. skipping file/, e.first) #'
+      assert_equal([], r)
+      assert_equal("foo\nbar\nbaz\n", File.read(t.path))
     end
   end
 
@@ -252,8 +247,6 @@ class TestArgf < Test::Unit::TestCase
   end
 
   def test_inplace_stdin
-    t = make_tempfile
-
     assert_in_out_err(["-", "-"], <<-INPUT, [], /Can't do inplace edit for stdio; skipping/)
       ARGF.inplace_mode = '.bak'
       f = ARGF.dup
@@ -264,8 +257,6 @@ class TestArgf < Test::Unit::TestCase
   end
 
   def test_inplace_stdin2
-    t = make_tempfile
-
     assert_in_out_err(["-"], <<-INPUT, [], /Can't do inplace edit for stdio/)
       ARGF.inplace_mode = '.bak'
       while line = ARGF.gets
@@ -424,11 +415,11 @@ class TestArgf < Test::Unit::TestCase
       end
     end
 
-    t1 = Tempfile.new("argf-foo")
+    t1 = open("#{@tmpdir}/argf-hoge", "w")
     t1.binmode
     t1.puts "foo"
     t1.close
-    t2 = Tempfile.new("argf-bar")
+    t2 = open("#{@tmpdir}/argf-moge", "w")
     t2.binmode
     t2.puts "bar"
     t2.close
@@ -453,6 +444,16 @@ class TestArgf < Test::Unit::TestCase
     end
   end
 
+  def test_read2_with_not_empty_buffer
+    ruby('-e', <<-SRC, @t1.path, @t2.path, @t3.path) do |f|
+      s = "0123456789"
+      ARGF.read(8, s)
+      p s
+    SRC
+      assert_equal("\"1\\n2\\n3\\n4\\n\"\n", f.read)
+    end
+  end
+
   def test_read3
     ruby('-e', <<-SRC, @t1.path, @t2.path, @t3.path) do |f|
       nil while ARGF.gets
@@ -470,6 +471,8 @@ class TestArgf < Test::Unit::TestCase
         loop do
           s << ARGF.readpartial(1)
           t = ""; ARGF.readpartial(1, t); s << t
+          # not empty buffer
+          u = "abcdef"; ARGF.readpartial(1, u); s << u
         end
       rescue EOFError
         puts s
@@ -501,6 +504,17 @@ class TestArgf < Test::Unit::TestCase
     end
   end
 
+  def test_readpartial_eof_twice
+    ruby('-W1', '-e', <<-SRC, @t1.path) do |f|
+      $stderr = $stdout
+      print ARGF.readpartial(256)
+      ARGF.readpartial(256) rescue p($!.class)
+      ARGF.readpartial(256) rescue p($!.class)
+    SRC
+      assert_equal("1\n2\nEOFError\nEOFError\n", f.read)
+    end
+  end
+
   def test_getc
     ruby('-e', <<-SRC, @t1.path, @t2.path, @t3.path) do |f|
       s = ""
@@ -676,6 +690,49 @@ class TestArgf < Test::Unit::TestCase
     end
   end
 
+  def test_skip_in_each_line
+    ruby('-e', <<-SRC, @t1.path, @t2.path, @t3.path) do |f|
+      ARGF.each_line {|l| print l; ARGF.skip}
+    SRC
+      assert_equal("1\n3\n5\n", f.read, '[ruby-list:49185]')
+    end
+    ruby('-e', <<-SRC, @t1.path, @t2.path, @t3.path) do |f|
+      ARGF.each_line {|l| ARGF.skip; puts [l, ARGF.gets].map {|s| s ? s.chomp : s.inspect}.join("+")}
+    SRC
+      assert_equal("1+3\n4+5\n6+nil\n", f.read, '[ruby-list:49185]')
+    end
+  end
+
+  def test_skip_in_each_byte
+    ruby('-e', <<-SRC, @t1.path, @t2.path, @t3.path) do |f|
+      ARGF.each_byte {|l| print l; ARGF.skip}
+    SRC
+      assert_equal("135".unpack("C*").join(""), f.read, '[ruby-list:49185]')
+    end
+  end
+
+  def test_skip_in_each_char
+    [[@t1, "\u{3042}"], [@t2, "\u{3044}"], [@t3, "\u{3046}"]].each do |f, s|
+      File.write(f.path, s, mode: "w:utf-8")
+    end
+    ruby('-Eutf-8', '-e', <<-SRC, @t1.path, @t2.path, @t3.path) do |f|
+      ARGF.each_char {|l| print l; ARGF.skip}
+    SRC
+      assert_equal("\u{3042 3044 3046}", f.read, '[ruby-list:49185]')
+    end
+  end
+
+  def test_skip_in_each_codepoint
+    [[@t1, "\u{3042}"], [@t2, "\u{3044}"], [@t3, "\u{3046}"]].each do |f, s|
+      File.write(f.path, s, mode: "w:utf-8")
+    end
+    ruby('-Eutf-8', '-Eutf-8', '-e', <<-SRC, @t1.path, @t2.path, @t3.path) do |f|
+      ARGF.each_codepoint {|l| printf "%x:", l; ARGF.skip}
+    SRC
+      assert_equal("3042:3044:3046:", f.read, '[ruby-list:49185]')
+    end
+  end
+
   def test_close
     ruby('-e', <<-SRC, @t1.path, @t2.path, @t3.path) do |f|
       ARGF.close
@@ -753,8 +810,7 @@ class TestArgf < Test::Unit::TestCase
     end
     argf = ARGF.class.new(*paths)
     paths.each do |path|
-      e = assert_raise(Errno::ENOENT) {argf.gets}
-      assert_match(/- #{Regexp.quote(path)}\z/, e.message)
+      assert_raise_with_message(Errno::ENOENT, /- #{Regexp.quote(path)}\z/) {argf.gets}
     end
     assert_nil(argf.gets, bug4274)
   end
@@ -763,4 +819,46 @@ class TestArgf < Test::Unit::TestCase
     bug5952 = '[ruby-dev:45160]'
     assert_ruby_status(["-e", "2.times {STDIN.tty?; readlines}"], "", bug5952)
   end
+
+  def test_lines
+    ruby('-W1', '-e', <<-SRC, @t1.path, @t2.path, @t3.path) do |f|
+      $stderr = $stdout
+      s = []
+      ARGF.lines {|l| s << l }
+      p s
+    SRC
+      assert_match(/deprecated/, f.gets)
+      assert_equal("[\"1\\n\", \"2\\n\", \"3\\n\", \"4\\n\", \"5\\n\", \"6\\n\"]\n", f.read)
+    end
+  end
+
+  def test_bytes
+    ruby('-W1', '-e', <<-SRC, @t1.path, @t2.path, @t3.path) do |f|
+      $stderr = $stdout
+      print Marshal.dump(ARGF.bytes.to_a)
+    SRC
+      assert_match(/deprecated/, f.gets)
+      assert_equal([49, 10, 50, 10, 51, 10, 52, 10, 53, 10, 54, 10], Marshal.load(f.read))
+    end
+  end
+
+  def test_chars
+    ruby('-W1', '-e', <<-SRC, @t1.path, @t2.path, @t3.path) do |f|
+      $stderr = $stdout
+      print [Marshal.dump(ARGF.chars.to_a)].pack('m')
+    SRC
+    assert_match(/deprecated/, f.gets)
+    assert_equal(["1", "\n", "2", "\n", "3", "\n", "4", "\n", "5", "\n", "6", "\n"], Marshal.load(f.read.unpack('m').first))
+    end
+  end
+
+  def test_codepoints
+    ruby('-W1', '-e', <<-SRC, @t1.path, @t2.path, @t3.path) do |f|
+      $stderr = $stdout
+      print Marshal.dump(ARGF.codepoints.to_a)
+    SRC
+      assert_match(/deprecated/, f.gets)
+      assert_equal([49, 10, 50, 10, 51, 10, 52, 10, 53, 10, 54, 10], Marshal.load(f.read))
+    end
+  end
 end
