@@ -1,3 +1,4 @@
+# coding: US-ASCII
 require 'test/unit'
 
 class TestPack < Test::Unit::TestCase
@@ -180,6 +181,7 @@ class TestPack < Test::Unit::TestCase
     assert_equal a[0], a.pack("p").unpack("p")[0]
     assert_equal a, a.pack("p").freeze.unpack("p*")
     assert_raise(ArgumentError) { (a.pack("p") + "").unpack("p*") }
+    assert_raise(ArgumentError) { (a.pack("p") << "d").unpack("p*") }
   end
 
   def test_format_string_modified
@@ -280,6 +282,9 @@ class TestPack < Test::Unit::TestCase
     assert_equal(["1"], "\x80".unpack("B1"))
     assert_equal(["10"], "\x80".unpack("B2"))
     assert_equal(["100"], "\x80".unpack("B3"))
+
+    assert_equal(Encoding::US_ASCII, "\xff\x00".unpack("b*")[0].encoding)
+    assert_equal(Encoding::US_ASCII, "\xff\x00".unpack("B*")[0].encoding)
   end
 
   def test_pack_unpack_hH
@@ -320,6 +325,9 @@ class TestPack < Test::Unit::TestCase
     assert_equal(["10e"], "\x10\xef".unpack("H3"))
     assert_equal(["10ef"], "\x10\xef".unpack("H4"))
     assert_equal(["10ef"], "\x10\xef".unpack("H5"))
+
+    assert_equal(Encoding::US_ASCII, "\x10\xef".unpack("h*")[0].encoding)
+    assert_equal(Encoding::US_ASCII, "\x10\xef".unpack("H*")[0].encoding)
   end
 
   def test_pack_unpack_cC
@@ -396,8 +404,17 @@ class TestPack < Test::Unit::TestCase
     assert_equal([578437695752307201, -506097522914230529], s2.unpack("q*"))
     assert_equal([578437695752307201, 17940646550795321087], s1.unpack("Q*"))
 
+    # Note: q! and Q! should not work on platform which has no long long type.
+    # Is there a such platform now?
+    s1 = [578437695752307201, -506097522914230529].pack("q!*")
+    s2 = [578437695752307201, 17940646550795321087].pack("Q!*")
+    assert_equal([578437695752307201, -506097522914230529], s2.unpack("q!*"))
+    assert_equal([578437695752307201, 17940646550795321087], s1.unpack("Q!*"))
+
     assert_equal(8, [1].pack("q").bytesize)
     assert_equal(8, [1].pack("Q").bytesize)
+    assert_operator(8, :<=, [1].pack("q!").bytesize)
+    assert_operator(8, :<=, [1].pack("Q!").bytesize)
   end
 
   def test_pack_unpack_nN
@@ -429,7 +446,7 @@ class TestPack < Test::Unit::TestCase
       %w(f d e E g G).each do |f|
         v = [x].pack(f).unpack(f)
         if x.nan?
-          assert(v.first.nan?)
+          assert_predicate(v.first, :nan?)
         else
           assert_equal([x], v)
         end
@@ -496,6 +513,10 @@ class TestPack < Test::Unit::TestCase
     assert_equal("M86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A\n!80``\n", ["a"*46].pack("u0"))
     assert_equal("M86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A\n!80``\n", ["a"*46].pack("u1"))
     assert_equal("M86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A\n!80``\n", ["a"*46].pack("u2"))
+    assert_equal(<<EXPECTED, ["a"*80].pack("u68"))
+_86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A
+186%A86%A86%A86%A86%A86$`
+EXPECTED
 
     assert_equal([""], "".unpack("u"))
     assert_equal(["a"], "!80``\n".unpack("u"))
@@ -525,6 +546,18 @@ class TestPack < Test::Unit::TestCase
     assert_equal(["\377"], "/w==\n".unpack("m"))
     assert_equal(["\377\377"], "//8=\n".unpack("m"))
     assert_equal(["\377\377\377"], "////\n".unpack("m"))
+    assert_equal([""], "A\n".unpack("m"))
+    assert_equal(["\0"], "AA\n".unpack("m"))
+    assert_equal(["\0"], "AA=\n".unpack("m"))
+    assert_equal(["\0\0"], "AAA\n".unpack("m"))
+
+    bug10019 = '[ruby-core:63604] [Bug #10019]'
+    size = ((4096-4)/4*3+1)
+    assert_separately(%W[- #{size} #{bug10019}], <<-'end;')
+      size = ARGV.shift.to_i
+      bug = ARGV.shift
+      assert_equal(size, ["a"*size].pack("m#{size+2}").unpack("m")[0].size, bug)
+    end;
   end
 
   def test_pack_unpack_m0
@@ -568,10 +601,18 @@ class TestPack < Test::Unit::TestCase
     assert_equal(["a"*1023], (("a"*73+"=\n")*14+"a=\n").unpack("M"))
     assert_equal(["\x0a"], "=0a=\n".unpack("M"))
     assert_equal(["\x0a"], "=0A=\n".unpack("M"))
-    assert_equal([""], "=0Z=\n".unpack("M"))
-    assert_equal([""], "=\r\n".unpack("M"))
+    assert_equal(["=0Z=\n"], "=0Z=\n".unpack("M"))
     assert_equal([""], "=\r\n".unpack("M"))
     assert_equal(["\xC6\xF7"], "=C6=F7".unpack('M*'))
+
+    assert_equal(["pre123after"], "pre=31=32=33after".unpack("M"))
+    assert_equal(["preafter"], "pre=\nafter".unpack("M"))
+    assert_equal(["preafter"], "pre=\r\nafter".unpack("M"))
+    assert_equal(["pre="], "pre=".unpack("M"))
+    assert_equal(["pre=\r"], "pre=\r".unpack("M"))
+    assert_equal(["pre=hoge"], "pre=hoge".unpack("M"))
+    assert_equal(["pre==31after"], "pre==31after".unpack("M"))
+    assert_equal(["pre===31after"], "pre===31after".unpack("M"))
   end
 
   def test_pack_unpack_P2
@@ -612,28 +653,6 @@ class TestPack < Test::Unit::TestCase
     assert_equal([0x100000000], "\220\200\200\200\000".unpack("w"), [0x100000000])
   end
 
-
-  def test_pack_unpack_M
-    assert_equal(["pre123after"], "pre=31=32=33after".unpack("M"))
-    assert_equal(["preafter"], "pre=\nafter".unpack("M"))
-    assert_equal(["preafter"], "pre=\r\nafter".unpack("M"))
-    assert_equal(["pre="], "pre=".unpack("M"))
-    assert_equal(["pre=\r"], "pre=\r".unpack("M"))
-    assert_equal(["pre=hoge"], "pre=hoge".unpack("M"))
-    assert_equal(["pre==31after"], "pre==31after".unpack("M"))
-    assert_equal(["pre===31after"], "pre===31after".unpack("M"))
-  end
-
-  def test_modify_under_safe4
-    s = "foo"
-    assert_raise(SecurityError) do
-      Thread.new do
-        $SAFE = 4
-        s.clear
-      end.join
-    end
-  end
-
   def test_length_too_big
     assert_raise(RangeError) { [].pack("C100000000000000000000") }
   end
@@ -651,4 +670,43 @@ class TestPack < Test::Unit::TestCase
     assert_nil("".unpack("i") {|x| result = x}, bug4059)
     assert_equal(:ok, result)
   end
+
+  def test_pack_garbage
+    verbose = $VERBOSE
+    $VERBOSE = false
+
+    assert_silent do
+      assert_equal "\000", [0].pack("*U")
+    end
+
+    $VERBOSE = true
+
+    _, err = capture_io do
+      assert_equal "\000", [0].pack("*U")
+    end
+
+    assert_match %r%unknown pack directive '\*' in '\*U'$%, err
+  ensure
+    $VERBOSE = verbose
+  end
+
+  def test_unpack_garbage
+    verbose = $VERBOSE
+    $VERBOSE = false
+
+    assert_silent do
+      assert_equal [0], "\000".unpack("*U")
+    end
+
+    $VERBOSE = true
+
+    _, err = capture_io do
+      assert_equal [0], "\000".unpack("*U")
+    end
+
+    assert_match %r%unknown unpack directive '\*' in '\*U'$%, err
+  ensure
+    $VERBOSE = verbose
+  end
+
 end
